/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMHTMLElement.idl"

/**
 * The nsIDOMHTMLDataListElement interface is the interface to a HTML
 * <datalist> element.
 *
 * For more information on this interface, please see
 * http://www.whatwg.org/specs/web-apps/current-work/#the-datalist-element
 *
 * @status UNDER_DEVELOPMENT
 */

interface nsIDOMHTMLCollection;

[scriptable, uuid(528e6a6b-f957-42e1-8d1b-eeeb2fd0b128)]
interface nsIDOMHTMLDataListElement : nsIDOMHTMLElement
{
  readonly attribute nsIDOMHTMLCollection options;
};

