/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMMozConnection;

[scriptable, uuid(c1685d27-f2e2-4ed9-998f-ff5b1442058f)]
interface nsIDOMMozNavigatorNetwork : nsISupports
{
  readonly attribute nsIDOMMozConnection mozConnection;
};
