/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:set ts=2 sw=2 sts=2 et cindent: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIWebBrowserChrome.idl"

/**
 * nsIWebBrowserChrome2 is an extension to nsIWebBrowserChrome.
 */
[scriptable, uuid(2585a7b1-7b47-43c4-bf17-c6bf84e09b7b)]
interface nsIWebBrowserChrome2 : nsIWebBrowserChrome
{
  /**
   * Called when the status text in the chrome needs to be updated.  This
   * method may be called instead of nsIWebBrowserChrome::SetStatus.  An
   * implementor of this method, should still implement SetStatus.
   *
   * @param statusType
   *        Indicates what is setting the text.
   * @param status
   *        Status string.  Null is an acceptable value meaning no status.
   * @param contextNode 
   *        An object that provides context pertaining to the status type.
   *        If statusType is STATUS_LINK, then statusContext may be a DOM
   *        node corresponding to the source of the link.  This value can
   *        be null if there is no context.
   */
  void setStatusWithContext(in unsigned long statusType,
                            in AString statusText,
                            in nsISupports statusContext);
};
